@@ -15,26 +15,24 @@ fn main() {
         .unwrap();
 
     let hello_world1 = delegate_loader
-        .get_function_pointer(
+        .get_function::<fn()>(
             pdcstr!("ExampleProject.Program, ExampleProject"),
             pdcstr!("HelloWorld1"),
             pdcstr!("ExampleProject.Program+HelloWorld1Delegate, ExampleProject"),
         )
         .unwrap();
-    let hello_world1: unsafe extern "system" fn() = unsafe { std::mem::transmute(hello_world1) };
-    unsafe { hello_world1() };
+    hello_world1();
 
     let hello_world2 = delegate_loader
-        .get_function_pointer_for_unmanaged_callers_only_method(
+        .get_function_with_unmanaged_callers_only::<fn()>(
             pdcstr!("ExampleProject.Program, ExampleProject"),
             pdcstr!("HelloWorld2"),
         )
         .unwrap();
-    let hello_world2: unsafe extern "system" fn() = unsafe { std::mem::transmute(hello_world2) };
-    unsafe { hello_world2() };
+    hello_world2();
 
     let hello_world3 = delegate_loader
-        .get_function_pointer_with_default_signature(
+        .get_function_with_default_signature(
             pdcstr!("ExampleProject.Program, ExampleProject"),
             pdcstr!("HelloWorld3"),
         )
