@@ -1,7 +1,6 @@
-use std::{
-    convert::TryFrom,
-    ops::{ControlFlow, FromResidual, Try},
-};
+use std::convert::TryFrom;
+#[cfg(feature = "nightly")]
+use std::ops::{ControlFlow, FromResidual, Try};
 
 use crate::bindings;
 use derive_more::{Deref, Display, From};
@@ -117,6 +116,7 @@ impl From<HostingError> for HostingResult {
     }
 }
 
+#[cfg(feature = "nightly")]
 impl Try for HostingResult {
     type Output = HostingSuccess;
     type Residual = HostingError;
@@ -131,6 +131,7 @@ impl Try for HostingResult {
     }
 }
 
+#[cfg(feature = "nightly")]
 impl FromResidual for HostingResult {
     fn from_residual(r: HostingError) -> Self {
         HostingResult(Err(r))
