@@ -6,3 +6,6 @@ pub use context::*;
 
 mod delegate_loader;
 pub use delegate_loader::*;
+
+mod managed_function;
+pub use managed_function::*;
