@@ -1,4 +1,4 @@
-use netcorehost::{hostfxr::GetFunctionPointerError, nethost, pdcstr};
+use netcorehost::{hostfxr::GetManagedFunctionError, nethost, pdcstr};
 
 #[path = "common.rs"]
 mod common;
@@ -18,67 +18,65 @@ fn get_function_pointer() {
         .get_delegate_loader_for_assembly(pdcstr!("tests/Test/bin/Debug/net5.0/Test.dll"))
         .unwrap();
 
-    let invalid_method_name = fn_loader.get_function_pointer_with_default_signature(
+    let invalid_method_name = fn_loader.get_function_with_default_signature(
         pdcstr!("Test.Program, Test"),
         pdcstr!("SomeMethodThatDoesNotExist"),
     );
     assert!(invalid_method_name.is_err());
     assert_eq!(
         unsafe { invalid_method_name.unwrap_err_unchecked() },
-        GetFunctionPointerError::MissingMethod
+        GetManagedFunctionError::MissingMethod
     );
 
-    let invalid_method_signature = fn_loader.get_function_pointer_with_default_signature(
-        pdcstr!("Test.Program, Test"),
-        pdcstr!("Main"),
-    );
+    let invalid_method_signature = fn_loader
+        .get_function_with_default_signature(pdcstr!("Test.Program, Test"), pdcstr!("Main"));
     assert!(invalid_method_signature.is_err());
     assert_eq!(
         unsafe { invalid_method_signature.unwrap_err_unchecked() },
-        GetFunctionPointerError::MissingMethod
+        GetManagedFunctionError::MissingMethod
     );
 
-    let invalid_type_name = fn_loader.get_function_pointer_with_default_signature(
+    let invalid_type_name = fn_loader.get_function_with_default_signature(
         pdcstr!("Test.SomeTypeThatDoesNotExist, Test"),
         pdcstr!("Hello"),
     );
     assert!(invalid_type_name.is_err());
     assert_eq!(
         unsafe { invalid_type_name.unwrap_err_unchecked() },
-        GetFunctionPointerError::TypeNotFound
+        GetManagedFunctionError::TypeNotFound
     );
 
-    let invalid_namespace_name = fn_loader.get_function_pointer_with_default_signature(
+    let invalid_namespace_name = fn_loader.get_function_with_default_signature(
         pdcstr!("SomeNamespaceThatDoesNotExist.Program, Test"),
         pdcstr!("Hello"),
     );
     assert!(invalid_namespace_name.is_err());
     assert_eq!(
         unsafe { invalid_namespace_name.unwrap_err_unchecked() },
-        GetFunctionPointerError::TypeNotFound
+        GetManagedFunctionError::TypeNotFound
     );
 
-    let invalid_assembly_name = fn_loader.get_function_pointer_with_default_signature(
+    let invalid_assembly_name = fn_loader.get_function_with_default_signature(
         pdcstr!("Test.Program, SomeAssemblyThatDoesNotExist"),
         pdcstr!("Hello"),
     );
     assert!(invalid_assembly_name.is_err());
     assert_eq!(
         unsafe { invalid_assembly_name.unwrap_err_unchecked() },
-        GetFunctionPointerError::AssemblyNotFound
+        GetManagedFunctionError::AssemblyNotFound
     );
 
-    let method_not_marked = fn_loader.get_function_pointer_for_unmanaged_callers_only_method(
+    let method_not_marked = fn_loader.get_function_with_unmanaged_callers_only::<fn()>(
         pdcstr!("Test.Program, Test"),
         pdcstr!("Hello"),
     );
     assert!(method_not_marked.is_err());
     assert_eq!(
         unsafe { method_not_marked.unwrap_err_unchecked() },
-        GetFunctionPointerError::MethodNotUnmanagedCallersOnly
+        GetManagedFunctionError::MethodNotUnmanagedCallersOnly
     );
 
-    let invalid_delegate_type_name = fn_loader.get_function_pointer(
+    let invalid_delegate_type_name = fn_loader.get_function::<fn()>(
         pdcstr!("Test.Program, Test"),
         pdcstr!("Hello"),
         pdcstr!("Test.Program+SomeDelegateThatDoesNotExist, Test"),
@@ -86,7 +84,7 @@ fn get_function_pointer() {
     assert!(invalid_delegate_type_name.is_err());
     assert_eq!(
         unsafe { invalid_delegate_type_name.unwrap_err_unchecked() },
-        GetFunctionPointerError::TypeNotFound
+        GetManagedFunctionError::TypeNotFound
     );
 
     context.close().unwrap();
@@ -107,27 +105,23 @@ fn get_delegate_loader_for_assembly() {
     let fn_loader = context
         .get_delegate_loader_for_assembly(pdcstr!("tests/errors.rs"))
         .unwrap();
-    let invalid_assembly_path = fn_loader.get_function_pointer_with_default_signature(
-        pdcstr!("Test.Program, Test"),
-        pdcstr!("Hello"),
-    );
+    let invalid_assembly_path = fn_loader
+        .get_function_with_default_signature(pdcstr!("Test.Program, Test"), pdcstr!("Hello"));
     assert!(invalid_assembly_path.is_err());
     assert_eq!(
         unsafe { invalid_assembly_path.unwrap_err_unchecked() },
-        GetFunctionPointerError::AssemblyNotFound
+        GetManagedFunctionError::AssemblyNotFound
     );
 
     let fn_loader = context
         .get_delegate_loader_for_assembly(pdcstr!("PathThatDoesNotExist.dll"))
         .unwrap();
-    let non_existant_assembly_path = fn_loader.get_function_pointer_with_default_signature(
-        pdcstr!("Test.Program, Test"),
-        pdcstr!("Hello"),
-    );
+    let non_existant_assembly_path = fn_loader
+        .get_function_with_default_signature(pdcstr!("Test.Program, Test"), pdcstr!("Hello"));
     assert!(non_existant_assembly_path.is_err());
     assert_eq!(
         unsafe { non_existant_assembly_path.unwrap_err_unchecked() },
-        GetFunctionPointerError::AssemblyNotFound
+        GetManagedFunctionError::AssemblyNotFound
     );
 
     context.close().unwrap();
