@@ -1,4 +1,4 @@
-use netcorehost::{cast_managed_fn, nethost, pdcstr};
+use netcorehost::{nethost, pdcstr};
 
 #[path = "common.rs"]
 mod common;
@@ -18,12 +18,11 @@ fn unmanaged_caller_hello_world() {
         .get_delegate_loader_for_assembly(pdcstr!("tests/Test/bin/Debug/net5.0/Test.dll"))
         .unwrap();
     let hello = fn_loader
-        .get_function_pointer_for_unmanaged_callers_only_method(
+        .get_function_with_unmanaged_callers_only::<fn() -> i32>(
             pdcstr!("Test.Program, Test"),
             pdcstr!("UnmanagedHello"),
         )
         .unwrap();
-    let hello = unsafe { cast_managed_fn!(hello, fn() -> i32) };
 
     let result = hello();
     assert_eq!(result, 42);
